# Python imports
import threading

# Lib imports

# Application imports
from libs.dto.lsp_messages import get_message_str
from libs.dto.lsp_message_structs import LSPResponseTypes, ClientRequest, ClientNotification
from .lsp_controller_stdin_stdout import LSPControllerSTDInSTDOut
from .lsp_controller_websocket import LSPControllerWebsocket



def _log_list():
    def add_log_entry(title: str, message: any):
        ...

    def clear():
        ...



# class LSPController(LSPControllerSTDInSTDOut):
class LSPController(LSPControllerWebsocket):
    def __init__(self):
        super(LSPController).__init__()

        self._language      = "python3"

        # https://github.com/microsoft/multilspy/tree/main/src/multilspy/language_servers
        # initialize-params-slim.json was created off of jedi_language_server one
        self._init_params   = settings_manager.get_lsp_init_data()

        self._start_command = None
        self._lsp_pid       = -1
        self._message_id    = 0

        self.read_lock      = threading.Lock()
        self.write_lock     = threading.Lock()
        self.log_list       = _log_list
        self.request_list   = {}

        self._subscribe_to_events()


    def _subscribe_to_events(self):
        event_system.subscribe("textDocument/didOpen",    self._lsp_did_open)
        event_system.subscribe("textDocument/didSave",    self._lsp_did_save)
        event_system.subscribe("textDocument/didClose",   self._lsp_did_close)
        event_system.subscribe("textDocument/didChange",  self._lsp_did_change)
        event_system.subscribe("textDocument/definition", self._lsp_definition)
        event_system.subscribe("textDocument/completion", self._lsp_completion)


    def set_language(self, language):
        self._language = language

    def set_log_list(self, log_list):
        self.log_list = log_list

    def set_start_command(self, start_command: list):
        self._start_command = start_command

    def unset_start_command(self):
        self._start_command = None

    def send_notification(self, method: str, params: {} = {}):
        self._send_message( ClientNotification(method, params) )

    def send_request(self, method: str, params: {} = {}):
        self._send_message( ClientRequest(self._message_id, method, params) )

        self.request_list[self._message_id] = {"method": method}
        self._message_id += 1

    def get_message_id(self) -> int:
        return self._message_id

    def start_stop_lsp(self):
        if self._lsp_pid == -1:
            pid = self.start_lsp()
            if not pid: return

            self._lsp_pid = pid
        else:
            self.stop_lsp()
            self.log_list.clear()

    def handle_lsp_response(self, lsp_response: LSPResponseTypes):
        self.log_list.add_log_entry("LSP Response", lsp_response)
        event_system.emit("respond-to-client", (get_message_str(lsp_response),))